#include <cstring>
#include <iostream>
#include <fstream>
#include <stdio.h>
#include <stdlib.h>
#include <algorithm>

#include "sbutils.h"
#include "sbwinazuredatastorage.h"
#include "sbmisc.h"

using namespace SecureBlackbox;

TElWinAzureDataStorage m_DataStorage (NULL);
TElHTTPSClient m_Client (NULL);
TElX509Certificate m_Cert (NULL);


void SB_CALLBACK DataStorageProgressEvent(void * /*_ObjectData*/, TObjectHandle /*Sender*/, TSBDataStorageOperationRaw Operation, int64_t Total, int64_t Current, int8_t * Cancel)
{
	int64_t percent = Current * 100 / Total;
	std::cout << percent << "%" << "\r";
}

void ShowContainerList()
{
    try
    {
		std::cout << "Container list:" << std::endl;

		TStringList conts;
		m_DataStorage.ListContainers(conts);
		for (int i = 0; i < conts.get_Count(); i++)
        {
			std::string container;
			conts.get_Strings(i, container);
			std::cout << container << std::endl;
        }
    }
    catch (SBException e)
    {
       std::cout << e.what() << std::endl;
    }
}

void ShowObjectList(std::string container)
{
	try
	{
		std::cout << "Object list for " << container << ":" << std::endl;

		TElDataStorageObjectList objs;
		m_DataStorage.List(container, objs);
		for (int i = 0; i < objs.get_Count(); i++)
		{
			TElCustomDataStorageObject* obj = objs.get_Objects(i);

			std::string name;
			obj->get_Name(name);

			std::cout << name << std::endl;
		}
	}
    catch (SBException e)
    {
       std::cout << e.what() << std::endl;
    }
}

std::string GetFilename(std::string const& path)
{
	std::string result;
	size_t pos = path.find_last_of("\\");
	if(pos != std::string::npos)
		result.assign(path.begin() + pos + 1, path.end());
	else
		result = path;

	return result;
}

void WriteObject(std::string containerName, std::string path)
{
	TFileStream inputStream (path, filemodeOpenRead);
    try
    {
        TElDefaultDataStorageSecurityHandler handler (NULL);
        TElMemoryCertStorage encryptionCerts (NULL);
        encryptionCerts.Add(m_Cert, false);
        handler.set_EncryptionCertificates(encryptionCerts);
        handler.set_KeyEncryptionAlgorithm(SB_ALGORITHM_CNT_AES128);
        handler.set_DataEncryptionAlgorithm(SB_ALGORITHM_CNT_AES128);
		handler.set_AuthInfoType(aitNone);

		std::cout << "Uploading object " << path  << std::endl;

		m_DataStorage.WriteObject(containerName, GetFilename(path), inputStream, handler);
    }
    catch (SBException e)
    {
		std::cout << "Object write error: " << e.what() << std::endl;
    }
}

void ReadObject(std::string containerName, std::string name, std::string path)
{
    try
    {
		TFileStream outputStream (path + name, filemodeCreate);

		std::cout << "Downloading object " << path + name << std::endl;
		m_DataStorage.ReadObject(containerName, name, outputStream);
    }
    catch (SBException e)
    {
		std::cout << "Error: " << e.what() << std::endl;
    }
}

std::string UnQuote(std::string s)
{
    if (s.length() < 1)
        return s;

	s.erase(remove( s.begin(), s.end(), '\"' ), s.end());

    return s;
}

bool LoadCertificate(std::string certFilename, std::string certPassword)
{
	int r = m_Cert.LoadFromFileAuto(certFilename, certPassword);
    if (r != 0)
    {
        std::cout << "Failed to load certificate, error " << r << std::endl;
    }
	return r == 0;
}

int main(int argc, char **argv)
{
	std::string helpMessage = "winazure.exe [flags] [operation] -c [filename] [password]" \
		"-w [container file]: upload file to storage" \
		"-r [container name path]: download file by 'name' in 'path' location" \
		"flags:" \
		"-containerlist : show container list;" \
		"-objectlist [container] : show object in container;" \
		"-c [filename] [password] : certificate";
	int i = 0;
	bool write = false;
	bool read = false;
	bool loadCert = false;
	std::string certFilename, certPassword;
	std::string container, filename, name, path;

	try
	{
		#error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
SetLicenseKey("<put the key here>");
	}
	catch (SBException E)
	{
		std::cout << "Failed to set license key:" << std::endl;
		std::cout << E.what() << std::endl;
	}

	std::cout << "EldoS Win Azure console demo." << std::endl;

	//SBDataStorage.Unit.DataStorageEncodingHandlersFactory().RegisterHandler(TElZlibDataStorageEncodingHandler.MetaClass.Instance);

    m_DataStorage.set_OnProgress(DataStorageProgressEvent, NULL);
    m_DataStorage.set_HTTPClient(m_Client);
    m_DataStorage.set_AccountName("devstoreaccount1");
    m_DataStorage.set_AccountKey("Eby8vdM02xNOcqFlqUwJPLlmEtlCDXJ1OUzFT50uSRZ6IFsuFq2UVErCz4I6tq/K1SZFPTOtr/KBHBeksoGMGw==");
    m_DataStorage.set_DevMode(true);

	try
	{
		i = 1;
		while (i < argc)
		{
			std::string param;
			param.assign(argv[i]);

			if (param == "-containerlist")
			{
				ShowContainerList();
			}
			else if (param == "-objectlist")
			{
				if (++i > argc)
					throw new SBException();

				container = UnQuote(argv[i]);	
				ShowObjectList(container);
			}
			else if (param == "-w")
			{
				if (++i > argc)
					throw new SBException();

				container = UnQuote(argv[i]);	

				if (++i > argc)
					throw new SBException();

				filename = UnQuote(argv[i]);

				write = true;
			}
			else if (param == "-r")
			{
				if (++i > argc)
					throw new SBException();

				container = UnQuote(argv[i]);	

				if (++i > argc)
					throw new SBException();

				name = UnQuote(argv[i]);

				if (++i > argc)
					throw new SBException();

				path = UnQuote(argv[i]);
			
				read = true;
			}
			else if (param == "-c")
			{
				if (++i > argc)
					throw new SBException();

				certFilename = UnQuote(argv[i]);

				if (++i < argc)
					certPassword = UnQuote(argv[i]);

				loadCert = true;
			}
			else if (param == "-h")
			{
				std::cout << helpMessage << std::endl;
			}

			i++;
		}	
	}
    catch (SBException E)
    {
		std::cout << "Parameters parsing error:" << E.what() << std::endl;
		std::cout << helpMessage << std::endl;
		return 0;
    }

	if (loadCert && LoadCertificate(certFilename, certPassword))
	{
		if (write)
			WriteObject(container, filename);
		else if (read)
			ReadObject(container, name, path);
	}

	//std::string result;
	//std::getline(std::cin, result);

	return 0;
}