#include <cstring>
#include <iostream>
#include <fstream>
#include <stdio.h>
#include <stdlib.h>
#include <algorithm>
#include <sstream>
#include <clocale>
#include <locale>
#include <string>
#include <vector>

#include "sbutils.h"
#include "sbpop3client.h"
#include "sbX509.h"
#include "sbmime.h"
#include "sbcertvalidator.h"

using namespace SecureBlackbox;

TElPOP3Client client (NULL);

std::string UnQuote(std::string s)
{
    if (s.length() < 1)
        return s;

	s.erase(remove( s.begin(), s.end(), '\"' ), s.end());

    return s;
}

inline std::string narrow(std::wstring const& text)
{
    std::locale const loc("");
    wchar_t const* from = text.c_str();
    std::size_t const len = text.size();
    std::vector<char> buffer(len + 1);
    std::use_facet<std::ctype<wchar_t> >(loc).narrow(from, from + len, '_', &buffer[0]);
    return std::string(&buffer[0], &buffer[len]);
}

void SB_CALLBACK CertificateValidateEvent(void * _ObjectData, TObjectHandle Sender, TElX509CertificateHandle X509Certificate, int8_t * Validate)
{
	TElX509Certificate Certificate(X509Certificate, false);
	TSBCertificateValidity Validity = cvInvalid;
	TSBCertificateValidityReason Reason;
	if ((Certificate.get_Chain() == NULL) || (Certificate.get_Chain()->get_Certificates(0)->getHandle() == X509Certificate))
	{
		// For proper CRL and OCSP validation please read instructions in 
		// description of ElX509CertificateValidator class in the help file
		std::time_t t = std::time(0);
		TElX509CertificateValidator CertificateValidator(NULL);
		CertificateValidator.ValidateForSSL(&Certificate, *static_cast<std::string*>(_ObjectData), "", hrServer, NULL, false, t, Validity, Reason);
		*Validate = (Validity == cvOk) || (Validity == cvSelfSigned);
		if (!(*Validate)) {
			std::cout << "There was a problem validating the server certificate. Do you wan't to continue? : Y or N" << std::endl;

			std::string input;
			std::getline(std::cin, input);
			std::transform(input.begin(), input.end(), input.begin(), ::toupper);

			*Validate = ((input == "Y") ? true : false);
		}
	}
	else
		*Validate = true;
}

int main(int argc, char **argv)
{
	int i;
	std::string helpMessage = "consolePOP3Client.exe [params]\n" \
		"[params]\n" \
		"-server : pop3 server;\n" \
		"-port : port;\n" \
		"-ssl : \"explicit\" or \"implicit\" mode;\n" \
		"-u : username;\n" \
		"-p : password;\n" \
		"-h : help;\n";
	int port = 0;
	bool useSLL = false;
	std::string server, username, password;
	TSBSSLMode sslMode = smExplicit;

	try
	{
		#error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
SetLicenseKey("<put the key here>");
	}
	catch (SBException E)
	{
		std::cout << "Failed to set license key:" << E.what() << std::endl;
	}

	std::cout << "EldoS POP3 client console demo." << std::endl;

    try
    {
		i = 1;
		while (i < argc)
		{
			std::string param;
			param.assign(argv[i]);

			if (param == "-server")
			{
                if (++i <= argc)
					server = UnQuote(argv[i]);				
			} else if (param == "-port")
			{
                if (++i <= argc)
				{
					std::string value = UnQuote(argv[i]);
					port = atoi(value.c_str());
				}
			} else if (param == "-ssl")
			{
                if (++i <= argc)
				{
					std::string value = UnQuote(argv[i]);
					if (value == "explicit")
					{
						sslMode = smExplicit;
						useSLL = true;
					} else if (value == "implicit")
					{
						sslMode = smImplicit;
						useSLL = true;
					} else
					{
						std::cout << "SSL/TLS mode incorrect:" << std::endl;
						return 0;
					}
				}
			} else if (param == "-u")
			{
                if (++i <= argc)
					username = UnQuote(argv[i]);				
			} else if (param == "-p")
			{
                if (++i <= argc)
					password = UnQuote(argv[i]);				
			} else if (param == "-h")
			{
				std::cout << helpMessage << std::endl;
				return 0;
			}

			i++;
		}
	}
	catch (SBException ex)
	{
		std::cout << "Parameters parsing error:" << ex.what() << std::endl;
		std::cout << helpMessage << std::endl;
		return 0;
	}

	if ((server == "") || (port == 0))
	{
		std::cout << "Set correct server name and port number" << std::endl;
		return 0;
	}

	try
	{
		client.set_Versions((TSBVersions)(f_sbTLS1 | f_sbTLS11 | f_sbTLS12));
		client.set_OnCertificateValidate(&CertificateValidateEvent, &server);
		client.set_Address(server);
		client.set_Port(port);
		client.set_UseSSL(useSLL);
		client.set_SSLMode(sslMode);

		int32_t count;
		int64_t size;

		client.Open();
		client.set_Username(username);
		client.set_Password(password);
		client.Login();
		client.GetMailboxInfo(count, size);

		std::cout << "You have " << count << " messages in the Inbox." << std::endl;
		std::cout << "Total messages size is " << size << " bytes." << std::endl; 
		std::cout << "Do you want to upload the headers? : Y or N" << std::endl;

		std::string input;
		std::getline(std::cin, input);
		std::transform(input.begin(), input.end(), input.begin(), ::toupper);

		if (input == "Y")
		{
			std::vector<TSBPOP3MessageID> IDs;
			client.GetMessageIDList(IDs);
			TElMessageHeader header;
			std::string charset = "";
			int64_t size;

			for (int i = 0; i < (int)IDs.size(); i++)
			{
				if (client.ReceiveMessageHeader(IDs[i].Index, charset, false, header) == SB_EL_OK)
				{
					size = client.GetMessageSize(IDs[i].Index);
					
					std::wstring wfrom, wsubject;

					TElMailAddressList Addresses (header.GetMailAddressList(L"From", 0), NULL);
					if (Addresses.get_Count() + Addresses.get_GroupsCount() != 1)
						wfrom = L"[Multiple addresses]";
					else if (Addresses.get_Count() == 1) {
						TElMailAddress addr (Addresses.GetAddress(0), NULL);
						if (addr.IsAlias())
							addr.get_Alias(wfrom);
						else
							addr.get_Address(wfrom);
					}
					else {
						TElMailAddressGroup addrGroup (Addresses.GetGroup(0), NULL); 
						addrGroup.get_Name(wfrom);
					}

					TElMessageHeaderField subjectField (header.GetField(L"Subject", 0), NULL);
					subjectField.get_Value(wsubject);					

					std::string from = narrow(wfrom);
					std::string subject = narrow(wsubject);
					
					std::cout << std::endl;
					std::cout << "Number\t" << i << std::endl;
					std::cout << "From\t" << from << std::endl;
					std::cout << "Subject\t" << subject << std::endl;
					std::cout << "Size\t" << size << " byte(s)" << std::endl;
				}
			}
		
		}		
	}
	catch (SBException E)
	{
		std::cout << "Error:" << E.what() << std::endl;
	}
}
