#include <cstring>
#include <iostream>
#include <stdlib.h>
#include <ctime>

#include "sbutils.h"

#include "sbpkcs11certstorage.h"
#include "sbpkcs11base.h"
#include "sbpkcs11common.h"

using namespace SecureBlackbox;

int main(int argc, char **argv) {

	try
	{
		#error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
SetLicenseKey("<put the key here>");
	}
	catch (SBException E)
	{
		std::cout << "Failed to set license key:" << std::endl;
		std::cout << E.what() << std::endl;
		return 0;
	}

	std::string paramLib;
	std::string paramPassword;
	int paramSlotIndex = 0;

	if (argc < 3)
	{
		std::cout << "Usage:" << std::endl;
		std::cout << "pkcs11storage -list -lib <path_to_pkcs11> -slot <slot_index> -pass <password>" << std::endl;
		return 0;
	}

	int i = 1;
	while (i < argc - 1)
	{
		char *arg = argv[i];
		if (!strcmp(arg, "-lib"))
			paramLib.assign(argv[++i]);
		else if (!strcmp(arg, "-slot"))
		{
			paramSlotIndex = atoi(argv[++i]);
		}
		else if (!strcmp(arg, "-pass") ||
			!strcmp(arg, "-password"))
			paramPassword.assign(argv[++i]);

		i++;
	}

	try
	{
		TElPKCS11CertStorage PKCS11CertStorage(NULL);
		PKCS11CertStorage.set_DLLName(paramLib);
		std::cout << "Opening PKCS11 storage" << std::endl;
		PKCS11CertStorage.Open();
		std::cout << "Select a slot:" << std::endl;
		for (int i = 0; i < PKCS11CertStorage.get_Module()->get_SlotCount(); i++)
		{
			TElPKCS11SlotInfo* pSlotInfo = PKCS11CertStorage.get_Module()->get_Slot(i);
			std::string desc;
			pSlotInfo->get_SlotDescription(desc);
			if (pSlotInfo->get_TokenPresent())
				std::cout << desc << " (token present)" << std::endl;
			else
				std::cout << desc << " (no token)" << std::endl;
		}

		int SlotIndex = paramSlotIndex;
		bool RO = PKCS11CertStorage.get_Module()->get_Slot(SlotIndex)->get_ReadOnly();
		TElPKCS11SessionInfoHandle hSession = SB_NULL_HANDLE;
		try
		{
			hSession = PKCS11CertStorage.OpenSession(SlotIndex, RO);
		}
		catch (SBException E)
		{
			if (RO)
				throw;

			// trying to open in readonly mode
			hSession = PKCS11CertStorage.OpenSession(SlotIndex, true);
		}

		if (hSession != NULL)
		{
			if (PKCS11CertStorage.get_Module()->get_Slot(SlotIndex)->PinNeeded())
			{
				TElPKCS11SessionInfo Session(hSession, false);
				Session.Login(utUser, paramPassword);
			}

			std::cout << "Certificates from PKCS11 storage:" << std::endl;
			for (int i = 0; i < PKCS11CertStorage.get_Count(); i++)
			{
				TElX509Certificate *pCert = PKCS11CertStorage.get_Certificates(i);
				// display certificate info
				TName name;
				pCert->get_SubjectName(name);
				std::cout << (i + 1) << "\tSubject: ";
				if (name.CommonName != NULL)
					std::cout << (char *)name.CommonName;
				else
					std::cout << (char *)name.Organization;

				std::cout << std::endl;
				pCert->get_IssuerName(name);
				std::cout << "\tIssuer: ";
				if (name.CommonName != NULL)
					std::cout << (char *)name.CommonName;
				else
					std::cout << (char *)name.Organization;

				std::cout << std::endl;
				std::vector<uint8_t> SerialNumber;
				pCert->get_SerialNumber(SerialNumber);
				std::string s;
				BinaryToString(SerialNumber, s);
				BeautifyBinaryString(s, ':', s);
				std::cout << "\tSerial number: " << s << std::endl;

				time_t t;
				t = (time_t)pCert->get_ValidFrom();
				char* dt = ctime(&t);
				if (dt != NULL)
					std::cout << "\tValid from: " << dt;

				t = (time_t)pCert->get_ValidTo();
				dt = ctime(&t);
				if (dt != NULL)
					std::cout << "\tValid to: " << dt;

				std::cout << "\tPrivate key: ";
				if (pCert->get_PrivateKeyExists())
					std::cout << "YES";
				else
					std::cout << "NO";

				std::cout << std::endl;
			}
		}

		hSession = NULL;
		PKCS11CertStorage.Close();
	}
	catch (SBException E)
	{
		std::cout << "Error: " << E.what() << std::endl;
	}

	return 0;
}
