#include <cstring>
#include <iostream>
#include <stdlib.h>

#include "sbsshcommon.h"
#include "sbsshkeystorage.h"
#include "sbsimplessh.h"
#include "sbutils.h"

using namespace SecureBlackbox;

void SB_CALLBACK Client_OnKeyValidate(void * /* objData */, TObjectHandle /* hSender */, TElSSHKeyHandle hServerKey, int8_t * Validate)
{
	try
	{
		TElSSHKey ServerKey(hServerKey, false);
		TMessageDigest128 M128;
		ServerKey.get_FingerprintMD5(M128);
		std::string dgst;
		DigestToStr(M128, true, dgst);
		std::cout << "Server key received, fingerprint " << dgst << std::endl;
		*Validate = true; // NEVER do this. You MUST check the key validity somehow
	}
	catch (SBException E)
	{
		std::cout << "Unexpected error in OnKeyValidate event handler" << std::endl;
		std::cout << E.what() << std::endl;
		std::cout << "Stack trace: " << E.getErrorStackTrace().c_str() << std::endl;
		*Validate = false;
	}
}

int main(int argc, char **argv) {

	try
	{
		#error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
SetLicenseKey("<put the key here>");
	}
	catch (SBException E)
	{
		std::cout << "Failed to set license key:" << std::endl;
		std::cout << E.what() << std::endl;
		return 0;
	}

	if (argc < 3)
	{
		std::cout << "Usage:" << std::endl;
		std::cout << "ConsoleSSH -addr[ess] <address> [-port <port>] -user[name] <user> -pass[word] <password> -command <command to execute remotely>" << std::endl;
		return 0;
	}

	int paramPort = 22;
	std::string paramCommand("ls -al");
	std::string paramAddress;
	std::string paramUsername;
	std::string paramPassword;

	int i = 1;
	while (i < argc - 1)
	{
		char *arg = argv[i];
		if (!strcmp(arg, "-command"))
			paramCommand.assign(argv[++i]);
		else if (!strcmp(arg, "-addr") ||
			!strcmp(arg, "-address"))
			paramAddress.assign(argv[++i]);
		else if (!strcmp(arg, "-port"))
		{
			paramPort = atoi(argv[++i]);
			if (0 == paramPort)
				paramPort = 22;
		}
		else if (!strcmp(arg, "-user") ||
			!strcmp(arg, "-username"))
			paramUsername.assign(argv[++i]);
		else if (!strcmp(arg, "-pass") ||
			!strcmp(arg, "-password"))
			paramPassword.assign(argv[++i]);

		i++;
	}

	TElSimpleSSHClient Client(NULL);
	Client.set_Address(paramAddress);
	Client.set_Port(paramPort);

	Client.set_Username(paramUsername);
	Client.set_Password(paramPassword);

	Client.set_OnKeyValidate(&Client_OnKeyValidate, NULL);

	try
	{
		std::cout << "Executing command '" << paramCommand << "' on " << paramAddress << ":" << paramPort << std::endl;

		std::vector<uint8_t> resp;
		Client.ExecuteCommand(paramCommand, resp);
		if (resp.size() > 0)
			std::cout.write((char *)&resp[0], resp.size());
	}
	catch (SBException E)
	{
		std::cout << "Connection failed due to exception:" << std::endl;
		std::cout << E.what() << std::endl;
		std::cout << "Stack trace: " << E.getErrorStackTrace().c_str() << std::endl;

		std::cout << "If you have ensured that all connection parameters are correct and you still can't connect," << std::endl;
		std::cout << "please contact EldoS support as described on http://www.eldos.com/sbb/support-tech.php" << std::endl;
		std::cout << "Remember to provide details about the error that happened." << std::endl;

		std::string sname = "";
		Client.get_ServerSoftwareName(sname);
		if (sname.length() > 0)
			std::cout << "Server software identified itself as: " << sname.c_str() << std::endl;
	}

	Client.~TElSimpleSSHClient();

	return 0;
}
